@@ -84,25 +84,13 @@ void free (void *ptr);
 #include <sys/vfs.h>
 #endif
 
-#ifdef __bsdi__
+/* 4.4BSD2 derived systems */
+#if defined(__bsdi__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
 #  ifndef MOUNT_UFS
 #    define xBSD
 #  endif
 #endif
 
-#ifdef __NetBSD__
-#   define xBSD
-#endif
-
-#ifdef __OpenBSD__
-#   define xBSD
-#endif
-
-#if defined(SCO_FLAVOR) && defined(__GNUC__)
-extern char* strdup(const char*);
-#endif
-
-char *strstr (const char *haystack, const char *needle);
 /* void error (void);  FIXME -- needed? */
 
 #ifdef DOLPHIN
